package com.gavin;

import java.util.ArrayList;
import java.util.List;

import com.gavin.rule.GavinTestRule;
import com.yindangu.v3.business.plugin.business.api.rule.IRule;
import com.yindangu.v3.plugin.vds.reg.api.IRegisterPlugin;
import com.yindangu.v3.plugin.vds.reg.api.builder.IComponentProfileBuilder;
import com.yindangu.v3.plugin.vds.reg.api.builder.IEntityBuilder;
import com.yindangu.v3.plugin.vds.reg.api.builder.IEntityBuilder.IFieldProfileBuilder;
import com.yindangu.v3.plugin.vds.reg.api.builder.IRuleBuilder;
import com.yindangu.v3.plugin.vds.reg.api.builder.IRuleBuilder.IRuleInputBuilder;
import com.yindangu.v3.plugin.vds.reg.api.builder.IRuleBuilder.IRuleOutputBuilder;
import com.yindangu.v3.plugin.vds.reg.api.model.IComponentProfileVo;
import com.yindangu.v3.plugin.vds.reg.api.model.IEntityProfileVo.IFieldProfileVo;
import com.yindangu.v3.plugin.vds.reg.api.model.IPluginProfileVo;
import com.yindangu.v3.plugin.vds.reg.api.model.IRuleProfileVo;
import com.yindangu.v3.plugin.vds.reg.api.model.IRuleProfileVo.IRuleInputProfileVo;
import com.yindangu.v3.plugin.vds.reg.api.model.IRuleProfileVo.IRuleOutputProfileVo;
import com.yindangu.v3.plugin.vds.reg.api.model.VariableType;
import com.yindangu.v3.plugin.vds.reg.common.RegVds;

/**
 * @author huangjr
 * @since 2021-05-28
 */
public class GavinTestPluginRegister implements IRegisterPlugin {

    /**
     * 返回构件元信息
     */
    @Override
    public IComponentProfileVo getComponentProfile() {
        //  构造构件元信息
        IComponentProfileBuilder componentProfileBuilder = RegVds.getPlugin().getComponentProfile();
        componentProfileBuilder.setGroupId("com.gavin");
        componentProfileBuilder.setCode("GavinTestVPluginComponent");      
        //componentProfileBuilder.setVersion("3.0.0");
        IComponentProfileVo componentProfileVo = componentProfileBuilder.build();

        return componentProfileVo;
    }

    /**
     * 返回插件元信息
     */
    @Override
    public List<IPluginProfileVo> getPluginProfile() {

        //  声明插件规则的元信息
        IRuleBuilder ruleBuilder = RegVds.getPlugin().getRulePlugin();            

        //  创建Gavin测试的规则插件元信息
        IRuleProfileVo gavinTestRuleProfileVo = createGavinTestRule(ruleBuilder);

        //  收集插件元信息集合
        List<IPluginProfileVo> pluginProfileLst = new ArrayList<IPluginProfileVo>();
        pluginProfileLst.add(gavinTestRuleProfileVo);

        return pluginProfileLst;
    }

    /**
     * 创建Gavin测试的规则插件元信息
     * @param ruleBuilder
     * @return
     */
    private IRuleProfileVo createGavinTestRule(IRuleBuilder ruleBuilder) {

        //  规则输入参数-字符串类型元信息
        IRuleInputProfileVo inputParamOne = 
            creatRuleInputProfileVo(ruleBuilder, "输入参数-字符串", GavinTestRule.TEST_STRING, VariableType.Char, "单值输入参数-" + GavinTestRule.TEST_STRING, "aaa", true, null);

        //  规则输入参数-字符串类型元信息
        IRuleInputProfileVo inputParamTwo = 
            creatRuleInputProfileVo(ruleBuilder, "输入参数-数值型", GavinTestRule.TEST_NUMBER, VariableType.Number, "单值输入参数-" + GavinTestRule.TEST_NUMBER, 123.45, false, null);

        //  实体字段构造器
        IEntityBuilder inputEntityBuilder = RegVds.getBuilder().getEntityProfileBuilder();  
        IFieldProfileVo oneField = creatFieldProfileVo(inputEntityBuilder, GavinTestRule.TEST_ENTITY_FIELD_ONE, "实体参数1", "实体第1个字段", VariableType.Char);
        IFieldProfileVo twoField = creatFieldProfileVo(inputEntityBuilder, GavinTestRule.TEST_ENTITY_FIELD_TWO, "实体参数2", "实体第2个字段", VariableType.Char);
        IFieldProfileVo threeField = creatFieldProfileVo(inputEntityBuilder, GavinTestRule.TEST_ENTITY_FIELD_THREE, "实体参数3", "实体第3个字段", VariableType.Char);

        //  构造实体字段
        List<IFieldProfileVo> fields = new ArrayList<IFieldProfileVo>();
        fields.add(oneField);
        fields.add(twoField);
        fields.add(threeField);

        //  规则输入参数-实体类型元信息
        IRuleInputProfileVo inputParamThree = 
            creatRuleInputProfileVo(ruleBuilder, "输入参数-实体类型", GavinTestRule.TEST_ENTITY, VariableType.Entity, "实体输入参数-" + GavinTestRule.TEST_ENTITY, null, false, fields);                                    

        //  添加规则输入参数元信息
        ruleBuilder.addInput(inputParamOne);
        ruleBuilder.addInput(inputParamTwo);
        ruleBuilder.addInput(inputParamThree);

        //  规则输出参数-字符串类型元信息
        IRuleOutputProfileVo outputParamOne = 
            creatRuleOutputProfileVo(ruleBuilder, GavinTestRule.TEST_OUTPARAM_ONE, "输出参数1", "单值输出参数-" + GavinTestRule.TEST_OUTPARAM_ONE, VariableType.Char, null);
        IRuleOutputProfileVo outputParamTwo = 
            creatRuleOutputProfileVo(ruleBuilder, GavinTestRule.TEST_OUTPARAM_TWO, "输出参数2", "单值输出参数-" + GavinTestRule.TEST_OUTPARAM_TWO, VariableType.Char, null);
        IRuleOutputProfileVo outputParamThree = 
            creatRuleOutputProfileVo(ruleBuilder, GavinTestRule.TEST_OUTPARAM_THREE, "输出参数3", "单值输出参数-" + GavinTestRule.TEST_OUTPARAM_THREE, VariableType.Char, null);

        //  规则输出参数-实体类型元信息
        IEntityBuilder outputEntityBuilder = RegVds.getBuilder().getEntityProfileBuilder();  
        IFieldProfileVo oneOutputField = creatFieldProfileVo(outputEntityBuilder, GavinTestRule.TEST_OUTPARAM_ENTITY_FIELD_ONE, "输出实体参数", "实体第1个字段", VariableType.Char);        
        List<IFieldProfileVo> outputFields = new ArrayList<IFieldProfileVo>();
        outputFields.add(oneOutputField);
        IRuleOutputProfileVo outputParamEntity = 
            creatRuleOutputProfileVo(ruleBuilder, GavinTestRule.TEST_OUTPARAM_ENTITY, "输出参数实体", "实体输出参数-" + GavinTestRule.TEST_OUTPARAM_ENTITY, VariableType.Entity, outputFields);            

        //  添加规则输出参数元信息
        ruleBuilder.addOutput(outputParamOne);
        ruleBuilder.addOutput(outputParamTwo);
        ruleBuilder.addOutput(outputParamThree);
        ruleBuilder.addOutput(outputParamEntity);

        //  创建规则插件
        IRuleProfileVo ruleProfileVo = createRuleProfileVo(ruleBuilder, GavinTestRule.class.getSimpleName(), "Gavin测试验证VPlugin", "Gavin测试验证VPlugin，包含输入输出参数", GavinTestRule.class, true);

        return ruleProfileVo;
    }

    /**
     * 创建规则插件
     * @param ruleBuilder
     * @param code
     * @param name
     * @param desc
     * @param ruleClass
     * @param transaction
     * @return
     */
    private IRuleProfileVo createRuleProfileVo(
            IRuleBuilder ruleBuilder, String code, String name, String desc, Class<? extends IRule> ruleClass, boolean transaction) {

        //  规则的编码，尽量保证唯一
        ruleBuilder.setCode(code);
        //  规则的中文名
        ruleBuilder.setName(name);
        //  规则的描述，通常填规则的用途和使用注意事项等
        ruleBuilder.setDesc(desc);
        //  规则的实现类
        ruleBuilder.setEntry(ruleClass);
        //  规则运行是否需要开启数据库事务
        ruleBuilder.setTransactionType(transaction);    
        
        return ruleBuilder.build();
    }

    /**
     * 构造规则输入参数
     * @param ruleBuilder
     * @param name
     * @param code
     * @param type
     * @param desc
     * @param defaultValue
     * @param required
     * @param fieldProfileVoLst
     * @return
     */
    private IRuleInputProfileVo creatRuleInputProfileVo(
            IRuleBuilder ruleBuilder, String name, String code, VariableType type, String desc, Object defaultValue, boolean required,
            List<IFieldProfileVo> fieldProfileVoLst) {

        IRuleInputBuilder ruleInputBuilder = ruleBuilder.newInput();
        //  参数类型为字符串
        ruleInputBuilder.setType(type);
        //  参数编码
        ruleInputBuilder.setCode(code);
        //  参数中文名
        ruleInputBuilder.setName(name);
        //  参数描述,通常填参数的用途和使用注意事项等
        ruleInputBuilder.setDesc(desc);
        //  参数默认值
        ruleInputBuilder.setDefault(defaultValue);
        //  参数是否为必填   
        ruleInputBuilder.setRequired(required);        
        if (fieldProfileVoLst != null && !fieldProfileVoLst.isEmpty()) {
            for (IFieldProfileVo fieldProfileVo : fieldProfileVoLst) {
                if (fieldProfileVo == null) {
                    continue;
                }
                ruleInputBuilder.addField(fieldProfileVo);
            }
        }
        
        IRuleInputProfileVo inputProfileVo = ruleInputBuilder.build();
        return inputProfileVo;
    }

    /**
     * 构造规则输出参数
     * @param ruleBuilder
     * @param code
     * @param name
     * @param desc
     * @param type
     * @param fieldProfileVoLst
     * @return
     */
    private IRuleOutputProfileVo creatRuleOutputProfileVo(
            IRuleBuilder ruleBuilder, String code, String name, String desc, VariableType type, List<IFieldProfileVo> fieldProfileVoLst) {
        IRuleOutputBuilder ruleOutputBuilder = ruleBuilder.newOutput();
        ruleOutputBuilder.setCode(code);
        ruleOutputBuilder.setName(name);
        ruleOutputBuilder.setDesc(desc);
        ruleOutputBuilder.setType(type);
        if (fieldProfileVoLst != null && !fieldProfileVoLst.isEmpty()) {
            for (IFieldProfileVo fieldProfileVo : fieldProfileVoLst) {
                if (fieldProfileVo == null) {
                    continue;
                }
                ruleOutputBuilder.addField(fieldProfileVo);
            }
        }
        IRuleOutputProfileVo outputProfileVo = ruleOutputBuilder.build();
        return outputProfileVo;
    }
    
    /**
     * 创建实体字段
     * @param code
     * @param name
     * @param desc
     * @param fieldType
     * @return
     */
    private IFieldProfileVo creatFieldProfileVo(IEntityBuilder inputEntityBuilder, String code, String name, String desc, VariableType fieldType) {
                
          //  构造新的字段1
          IFieldProfileBuilder fieldProfileBuilder = inputEntityBuilder.newField();
          fieldProfileBuilder.setCode(code);
          fieldProfileBuilder.setName(name);
          fieldProfileBuilder.setDesc(desc);
          fieldProfileBuilder.setType(fieldType);
          IFieldProfileVo fieldProfileVo = fieldProfileBuilder.build();        
          return fieldProfileVo;
    }

}